( cd usr/man/man3 ; rm -rf attr_list_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz attr_list_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf path_to_fshandle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz path_to_fshandle.3.gz )
( cd usr/man/man3 ; rm -rf handle_to_fshandle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz handle_to_fshandle.3.gz )
( cd usr/man/man3 ; rm -rf fd_to_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz fd_to_handle.3.gz )
( cd usr/man/man3 ; rm -rf getparents_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz getparents_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf readlink_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz readlink_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf fssetdm_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz fssetdm_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf open_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz open_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf attr_multi_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz attr_multi_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf getparentpaths_by_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz getparentpaths_by_handle.3.gz )
( cd usr/man/man3 ; rm -rf free_handle.3.gz )
( cd usr/man/man3 ; ln -sf path_to_handle.3.gz free_handle.3.gz )
( cd usr/man/man8 ; rm -rf xfs_info.8.gz )
( cd usr/man/man8 ; ln -sf xfs_growfs.8.gz xfs_info.8.gz )
( cd usr/lib64 ; rm -rf libhandle.so )
( cd usr/lib64 ; ln -sf /lib64/libhandle.so.1 libhandle.so )
( cd lib64 ; rm -rf libhandle.so.1 )
( cd lib64 ; ln -sf libhandle.so.1.0.3 libhandle.so.1 )
